"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const util = require("util");
const jsdom = require("jsdom");
const xjs = require("extrajs");
const Node_class_1 = require("./Node.class");
const DocumentFragment_class_1 = require("./DocumentFragment.class");
const HTMLTemplateElement_class_1 = require("./HTMLTemplateElement.class");
class xjs_Document extends Node_class_1.default {
    static async fromFile(filepath) {
        let data = await util.promisify(fs.readFile)(filepath, 'utf8');
        return new xjs_Document(new jsdom.JSDOM(data).window.document);
    }
    static fromFileSync(filepath) {
        let data = fs.readFileSync(filepath, 'utf8');
        return new xjs_Document(new jsdom.JSDOM(data).window.document);
    }
    constructor(node) {
        super(node);
    }
    get node() { return super.node; }
    prepend(...contents) {
        this.node.prepend(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    append(...contents) {
        this.node.append(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    innerHTML() {
        let root = this.node.querySelector('html');
        if (root === null)
            throw new ReferenceError('No <html> element found.');
        return new jsdom.JSDOM('<!doctype html>' + root.outerHTML).serialize();
    }
    importLinks(dirpath) {
        if (!('import' in jsdom.JSDOM.fragment('<link rel="import" href="https://example.com/"/>').querySelector('link'))) {
            console.warn('`HTMLLinkElement#import` is not yet supported. Replacing `<link>`s with their imported contents…');
            this.node.querySelectorAll('link[rel~="import"][data-import]').forEach((link) => {
                let imported = xjs.Object.switch(link.getAttribute('data-import'), {
                    'document': (lnk) => DocumentFragment_class_1.default.fromFileSync(path.resolve(dirpath, lnk.href)).node,
                    'template': (lnk) => HTMLTemplateElement_class_1.default.fromFileSync(path.resolve(dirpath, lnk.href)).content(),
                    'default': () => null,
                })(link);
                if (imported) {
                    link.after(imported);
                    link.remove();
                }
            });
        }
        return this;
    }
    async importLinksAsync(dirpath) {
        if (!('import' in jsdom.JSDOM.fragment('<link rel="import" href="https://example.com/"/>').querySelector('link'))) {
            console.warn('`HTMLLinkElement#import` is not yet supported. Replacing `<link>`s with their imported contents…');
            await Promise.all([...this.node.querySelectorAll('link[rel~="import"][data-import]')].map(async (link) => {
                let imported = await xjs.Object.switch(link.getAttribute('data-import'), {
                    'document': async (lnk) => (await DocumentFragment_class_1.default.fromFile(path.resolve(dirpath, lnk.href))).node,
                    'template': async (lnk) => (await HTMLTemplateElement_class_1.default.fromFile(path.resolve(dirpath, lnk.href))).content(),
                    'default': async () => null,
                })(link);
                if (imported) {
                    ;
                    link.after(imported);
                    link.remove();
                }
            }));
        }
        return this;
    }
}
exports.default = xjs_Document;
