"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const util = require("util");
const jsdom = require("jsdom");
const xjs = require("extrajs");
const Node_class_1 = require("./Node.class");
class xjs_DocumentFragment extends Node_class_1.default {
    static concat(...contents) {
        return new xjs_DocumentFragment(jsdom.JSDOM.fragment('')).append(...contents).innerHTML();
    }
    static async fromFile(filepath) {
        let data = await util.promisify(fs.readFile)(filepath, 'utf8');
        return new xjs_DocumentFragment(jsdom.JSDOM.fragment(data));
    }
    static fromFileSync(filepath) {
        let data = fs.readFileSync(filepath, 'utf8');
        return new xjs_DocumentFragment(jsdom.JSDOM.fragment(data));
    }
    constructor(node) {
        super(node);
    }
    get node() { return super.node; }
    prepend(...contents) {
        this.node.prepend(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    append(...contents) {
        this.node.append(...contents.map((c) => (c instanceof Node_class_1.default) ? c.node :
            (c === null) ? '' : c));
        return this;
    }
    innerHTML() {
        return [...this.node.childNodes].map((node) => xjs.Object.switch(`${node.nodeType}`, {
            [Node_class_1.default.NodeType.ELEMENT_NODE]: (el) => el.outerHTML,
            [Node_class_1.default.NodeType.TEXT_NODE]: (text) => text.data,
            [Node_class_1.default.NodeType.COMMENT_NODE]: (comm) => `<!--${comm.data}-->`,
            [Node_class_1.default.NodeType.DOCUMENT_FRAGMENT_NODE]: (frag) => new xjs_DocumentFragment(frag).innerHTML(),
            'default': () => null,
        })(node)).join('');
    }
    importLinks(dirpath) {
        const xjs_HTMLTemplateElement = require('./HTMLTemplateElement.class.js').default;
        if (!('import' in jsdom.JSDOM.fragment('<link rel="import" href="https://example.com/"/>').querySelector('link'))) {
            console.warn('`HTMLLinkElement#import` is not yet supported. Replacing `<link>`s with their imported contents…');
            this.node.querySelectorAll('link[rel~="import"][data-import]').forEach((link) => {
                let imported = xjs.Object.switch(link.getAttribute('data-import'), {
                    'document': (lnk) => xjs_DocumentFragment.fromFileSync(path.resolve(dirpath, lnk.href)).node,
                    'template': (lnk) => xjs_HTMLTemplateElement.fromFileSync(path.resolve(dirpath, lnk.href)).content(),
                    'default': () => null,
                })(link);
                if (imported) {
                    link.after(imported);
                    link.remove();
                }
            });
        }
        return this;
    }
    async importLinksAsync(dirpath) {
        const xjs_HTMLTemplateElement = require('./HTMLTemplateElement.class.js').default;
        if (!('import' in jsdom.JSDOM.fragment('<link rel="import" href="https://example.com/"/>').querySelector('link'))) {
            console.warn('`HTMLLinkElement#import` is not yet supported. Replacing `<link>`s with their imported contents…');
            await Promise.all([...this.node.querySelectorAll('link[rel~="import"][data-import]')].map(async (link) => {
                let imported = await xjs.Object.switch(link.getAttribute('data-import'), {
                    'document': async (lnk) => (await xjs_DocumentFragment.fromFile(path.resolve(dirpath, lnk.href))).node,
                    'template': async (lnk) => (await xjs_HTMLTemplateElement.fromFile(path.resolve(dirpath, lnk.href))).content(),
                    'default': async () => null,
                })(link);
                if (imported) {
                    ;
                    link.after(imported);
                    link.remove();
                }
            }));
        }
        return this;
    }
}
exports.default = xjs_DocumentFragment;
